# $Id: Title.pm,v 1.10 2002/02/09 14:26:34 joern Exp $

#-----------------------------------------------------------------------
# Copyright (C) 2001-2002 Jrn Reder <joern@zyn.de> All Rights Reserved
# 
# This program is part of Video::DVDRip, which is free software; you can
# redistribute it and/or modify it under the same terms as Perl itself.
#-----------------------------------------------------------------------

package Video::DVDRip::Cluster::Title;

use base Video::DVDRip::Title;

use Carp;
use strict;

use File::Basename;

sub chunks			{ shift->{chunks}			}
sub chunk_cnt			{ shift->{chunk_cnt}			}

sub chunks_finished		{ shift->{chunks_finished}		}
sub set_chunks_finished		{ shift->{chunks_finished}	= $_[1]	}

sub frames_finished		{ shift->{frames_finished}		}
sub set_frames_finished		{ shift->{frames_finished}	= $_[1]	}

sub frames_per_chunk		{ shift->{frames_per_chunk}		}
sub set_frames_per_chunk	{ shift->{frames_per_chunk}	= $_[1]	}

sub with_avisplit		{ shift->{with_avisplit} 		}
sub set_with_avisplit		{ shift->{with_avisplit} 	= $_[1] }

sub with_cleanup		{ shift->{with_cleanup} 		}
sub set_with_cleanup		{ shift->{with_cleanup} 	= $_[1] }


sub set_chunk_cnt {
	my $self = shift;
	my ($chunk_cnt) = @_;

	my @chunks = ( 'todo' ) x $chunk_cnt;

	$self->{chunk_cnt} = $chunk_cnt;
	$self->{chunks}    = \@chunks;

	$self->set_chunks_finished ( 0 );
	$self->set_frames_per_chunk ( int($self->frames / $chunk_cnt) );

	$self->log ("Chunk count for project '".$self->project->label."' set to $chunk_cnt");

	return $chunk_cnt;
}

sub set_chunk_state {
	my $self = shift;
	my %par = @_;
	my ($nr, $state) = @par{'nr','state'};
	
	croak "Illegal chunk number $nr" if $nr >= $self->chunk_cnt;

	$self->chunks->[$nr] = $state;
	
	$self->set_chunks_finished ( $self->chunks_finished + 1 )
		if $state eq 'finished';

	if ( $self->chunks_finished == $self->chunk_cnt ) {
		$self->project->set_state ('transcoded');
	}

	$self->project->save;

	1;
}

sub get_chunk_state {
	my $self = shift;
	my %par = @_;
	my ($nr) = @par{'nr'};
	
	croak "Illegal chunk number $nr" if $nr >= $self->chunk_cnt;
	
	return $self->chunks->[$nr];
}

sub get_next_chunk_nr {
	my $self = shift;
	
	my $chunk_cnt = $self->chunk_cnt;

	# find first chunk with 'todo' state
	for (my $i=0; $i < $chunk_cnt; ++$i ) {
		if ( $self->get_chunk_state ( nr => $i ) eq 'todo' ) {
			$self->set_chunk_state (
				nr    => $i,
				state => 'transcoding'
			);
			# and returns its number
			return $i;
		}
	}
	
	# return undef if we have no 'todo' chunks
	return;
}

sub reset_chunks {
	my $self = shift;
	
	my $chunk_cnt = $self->chunk_cnt;

	# reset all chunks with 'transcoding' state
	for (my $i=0; $i < $chunk_cnt; ++$i ) {
		if ( $self->get_chunk_state ( nr => $i ) eq 'transcoding' ) {
			$self->set_chunk_state (
				nr    => $i,
				state => 'todo'
			);
		}
	}
	
	1;
}

sub create_vob_dir {
	my $self = shift;
	
	# no vob_dir creating here. This is done just before we
	# start transcoding to prevent from too much ssh remote
	# communication.
	
	return 1;
}

sub create_avi_dir {
	my $self = shift;

	# no avi_dir creating here. This is done just before we
	# start transcoding to prevent from too much ssh remote
	# communication.
	
	return 1;
}

sub avi_file {
	my $self = shift; $self->trace_in;

	my $job      = $self->project->assigned_job or croak "No job assigned";
	my $chunk_nr = $job->chunk_nr;

	croak "No chunk assigned" if not defined $chunk_nr;

	return sprintf (
		"%s/%s-%03d-chunk-%05d.avi",
		$self->project->avi_dir,
		$self->project->name,
		$self->nr,
		$chunk_nr
	);
}

sub final_avi_file {
	my $self = shift;

	my $job      = $self->project->assigned_job or croak "No job assigned";
	my $chunk_nr = $job->chunk_nr;

	croak "No chunk assigned" if not defined $chunk_nr;
	
	return sprintf (
		"%s/%03d/cluster/%s-%03d-%05d.avi",
		$self->project->final_avi_dir,
		$self->nr,
		$self->project->name,
		$self->nr,
		$chunk_nr
	);
}

sub merged_avi_file {
	my $self = shift;

	return sprintf (
		"%s/%03d/cluster/%s-%03d-merged.AVI",
		$self->project->final_avi_dir,
		$self->nr,
		$self->project->name,
		$self->nr
	);
}

sub target_avi_file {
	my $self = shift;

	return sprintf (
		"%s/%03d/%s-%03d.avi",
		$self->project->final_avi_dir,
		$self->nr,
		$self->project->name,
		$self->nr
	);
}

sub multipass_log_dir {
	my $self = shift;
	return $self->project->avi_dir;
}

sub get_transcode_command {
	my $self = shift;

	my $job      = $self->project->assigned_job or croak "No job assigned";
	my $chunk_nr = $job->chunk_nr;

	croak "No chunk assigned" if not defined $chunk_nr;

	my $chunk_cnt = $self->chunk_cnt;
	my $nav_file  = $self->vob_nav_file;

	my $command   = $self->SUPER::get_transcode_command (@_);

	# no -c in cluster mode
	$command =~ s/ -c \d+-\d+//;

	# add -W option for chunk selection
	$command .= " -W $chunk_nr,$chunk_cnt,$nav_file";

	# add directory creation code
	my $avi_dir = dirname $self->avi_file;
	$command = "test -d '$avi_dir' || mkdir -p '$avi_dir'; $command";

	$command .= " && echo DVDRIP_SUCCESS";

	return $command;
}

sub get_move_avi_command {
	my $self = shift;

	my $avi_file       = $self->avi_file;
	my $final_avi_file = $self->final_avi_file;
	
	my $avi_dir = dirname $final_avi_file;

	return  "test -d '$avi_dir' ||  mkdir -p '$avi_dir'; ".
		"mv '$avi_file' '$final_avi_file'";
}

sub get_merge_command {
	my $self = shift;
	
	my $merged_avi_file = $self->merged_avi_file;
	my $avi_dir = dirname $merged_avi_file;
	
	my $command = "avimerge -o '$merged_avi_file' -i $avi_dir/*.avi";

	$command .= " && echo DVDRIP_SUCCESS";
	
	$command .= " && rm $avi_dir/*.avi" if $self->with_cleanup;

	return $command;
}

sub get_process_audio_command {
	my $self = shift;
	
	my $vob_dir           = $self->vob_dir;
	my $merged_avi_file   = $self->merged_avi_file;
	my $target_avi_file   = $self->target_avi_file;
	my $nav_file	      = $self->vob_nav_file;
	my $chunk_cnt	      = $self->chunk_cnt;
	my $audio_channel     = $self->audio_channel;
	my $avi_dir           = dirname $merged_avi_file;
	my $volume_rescale    = $self->tc_volume_rescale;

	my $command = 
		"transcode -p '$vob_dir/' ".
		"-a $audio_channel ".
		"-W$chunk_cnt,$chunk_cnt,$nav_file ".
		"-i '$merged_avi_file' ".
		"-o '$target_avi_file' ".
		"-s $volume_rescale ".
		"-P 1 -x avi,vob -y raw -u 50 ";

	$command .= " && echo DVDRIP_SUCCESS";
	$command .= " && rm $merged_avi_file" if $self->with_cleanup;

	return $command;
}

sub get_split_command {
	my $self = shift;
	my $avi_dir = dirname($self->target_avi_file);
	my $target_avi_file   = $self->target_avi_file;

	my $command =
		$self->SUPER::get_split_command.
		" && echo DVDRIP_SUCCESS && ".
		q[perl -e 'while (<].$avi_dir.q[/*avi-0*>) {].
		q[$from=$_; s/.avi-00(..)/q{-}.sprintf(qq{%02d},$1+1).qq{.avi}/e;].
		q[rename ($from, $_);}'];
	
	$command .= " && rm '$target_avi_file'" if $self->with_cleanup;
	
	return $command;
}

sub save {
	my $self = shift;
	
	$self->project->save;
	
	1;
}

1;
